@@ -167,7 +167,7 @@ def batch_import(self, batch_id, load_chunksize=5, placeholder_representation="%
         
         # make a chain from task signatures that churn through each enqueued object
         import_chain = celery.chain(
-            import_file.si(load.pk) for load in enqueued
+            import_file2.si(load.pk) for load in enqueued
             )
         
         # this could, alternatively, be a task group
@@ -181,6 +181,210 @@ def batch_import(self, batch_id, load_chunksize=5, placeholder_representation="%
         return True
 
 
+"""
+
+try:
+    module = importlib.import_module(x)
+    installed[x] = True
+except ImportError:
+    installed[x] = False
+
+"""
+
+
+@celery.shared_task(bind=True, ignore_result=True)
+def import_file2(self, importqueue_pk, insert_size=5, slice_size=1000):
+    load = ImportQueue.objects.get(pk=importqueue_pk)
+    load.celery_task_id = self.request.id
+    load.save()
+    
+    destination = load.batch.destination
+    similar_load_qs = ImportQueue.objects.filter(batch__destination=destination, file=load.file)
+    similar_completed = similar_load_qs.filter(complete__isnull=False)
+    similar_started = similar_load_qs.filter(start__isnull=False)
+    
+
+    if similar_completed.count() > 0:
+        #appears to already have been imported. skip this one
+        logger.warning('It looks like <%s> was already imported to <%s> at %s; skipping for now' % (load, destination, similar_completed[0].complete))
+        load.fail()
+        return False
+    
+    elif similar_started.count() > 0:
+        #appears to already have been imported. skip this one
+        logger.warning('It looks like an import was already started for importing <%s> to <%s> at %s; skipping for now' % (load, destination, similar_started[0].start))
+        load.fail()
+        return False
+
+    else:
+        # proceed with making the file happen
+        load.status = 'IN PROCESS'
+        load.start = datetime.datetime.utcnow().replace(tzinfo=utc)
+        load.save()
+        
+        logger.info('Begin import of %s to %s' % (load, destination))
+        
+        tls = destination.source.top_level_source.abbreviation
+        if tls == 'HCUP':
+            meta = pyhcup.sas.meta_from_sas(load.file.loadfile.full_path)
+            augmented = pyhcup.meta.augment(meta)
+            
+            # store the columns for future reference
+            for k, v in augmented.T.to_dict().iteritems():
+                #col_supplemental_meta = pyhcup.db.col_from_invalue(v['informat'])
+                col = Column(
+                    parent_file=load.file,
+                    name=v['field'],
+                    col_type=v['data_type'].upper(),
+                    col_scale=v['scale'],
+                    col_precision=v['length'],
+                    informat=v['informat'],
+                    )
+                col.save()
+        
+        elif tls == 'PUDF':
+            meta = pyhcup.tx.meta_from_txt(load.file.loadfile.full_path)
+            augmented = pyhcup.meta.augment(meta, dialect='PUDF')
+            
+            # store the columns for future reference
+            for k, v in augmented.T.to_dict().iteritems():
+                # col_supplemental_meta = pyhcup.db.col_from_invalue(v['informat'])
+                col = Column(
+                    parent_file=load.file,
+                    name=v['field'],
+                    col_type=v['data_type'].upper(),
+                    col_scale=v['scale'],
+                    col_precision=v['length'],
+                    # informat=v['informat'],
+                    )
+                col.save()
+                
+        table = destination.name
+        schema = destination.schema
+        state = load.file.state.abbreviation
+        year = load.file.year
+        dsabbr = destination.source.abbreviation
+        
+        # FROM HERE BELOW, SPLIT OFF PSYCOPG2 VS NOT
+        
+        # check to see if psycopg2 is how we're going
+        db_def = settings.DATABASES[HACHOIR_DB_ENTRY]
+        backend = db_def['ENGINE'].split('.')[-1]
+        
+        if 'psycopg2' in backend and dsabbr.upper() in ['CORE', 'BASE']:
+            # do it Rockapella
+            pg_import_file.delay(load.pk)
+            return True
+        
+        else:
+            # use the old chunky way
+            if load.file.file_type.compression == 'NONE':
+                # uncompressed
+                handle = open(load.file.full_path)
+            elif load.file.file_type.compression == 'ZIP':
+                # pkzip compression
+                handle = pyhcup.hachoir.openz(os.path.dirname(load.file.full_path), load.file.filename)
+            
+            # make a generator to yield one slice_size slice of the file in question at a time
+            reader = pyhcup.parser.read(handle, meta, chunksize=slice_size)
+            
+            # take 5
+            # loop through slices and dispatch chunker.chunks() for each
+            # this is preferred to a chain so that it doesn't wait to start until after all the signatures are generated
+            result_lst = []
+            for df in reader:
+                # everything we insert will always have a state and year associated with it
+                # if they aren't already in the frame, add them from the passed values
+                # (which should in turn be from the loadfile object)
+                if 'state' not in df.columns.map(lambda x: str(x).lower()):
+                    df['state'] = state
+                
+                if 'year' not in df.columns.map(lambda x: str(x).lower()):
+                    df['year'] = year
+                
+                cleaned_df = replace_sentinels(df)
+                
+                if dsabbr == 'CHGS' and not meta_is_long(meta):
+                    # this is a wide charges file
+                    # convert before chunking it out
+                    cleaned_df = df_wtl(cleaned_df, category='CHGS')
+                
+                result_lst.append(chunker(
+                        cleaned_df,
+                        table,
+                        schema,
+                        insert_size
+                        )
+                    )
+            
+            import_complete.apply_async((result_lst, load.pk), countdown=10)
+            return True
+
+
+@celery.shared_task(bind=True, ignore_result=True)
+def pg_import_file(self, importqueue_pk):
+    """Assumes file has been checked already for repeat import, meta matching, etc.
+    
+    Also assumes psycopg2 availability.
+    """
+    load = ImportQueue.objects.get(pk=importqueue_pk)
+    target_file = load.file
+    
+    c = target_file.file_type.compression
+    
+    if c == 'NONE':
+        # uncompressed
+        handle = open(target_file.full_path)
+    elif c == 'ZIP':
+        # pkzip compression
+        handle = pyhcup.hachoir.openz(os.path.dirname(target_file.full_path), target_file.filename)
+    
+    
+    destination = load.batch.destination
+    meta = pyhcup.sas.meta_from_sas(target_file.loadfile.full_path)
+    table = destination.name
+    schema = destination.schema
+    state = target_file.state.abbreviation
+    year = target_file.year
+    
+    
+    # establish connection directly with psycopg2
+    import psycopg2
+    db_def = settings.DATABASES[HACHOIR_DB_ENTRY]
+    cnxn = psycopg2.connect(
+        host=db_def['HOST'],
+        port=db_def['PORT'],
+        user=db_def['USER'],
+        password=db_def['PASSWORD'],
+        database=db_def['NAME'],
+        )
+    
+    # get helper functions
+    from pyhcup.db import pg_rawload, pg_staging, pg_shovel, pg_drop
+    
+    # let 'er rip
+    rawload = pg_rawload(cnxn, handle, meta)
+    staging = pg_staging(cnxn, rawload, meta, state, year)
+    
+    fields = [x for x in meta.field if x.lower() not in ['state', 'year']]
+    fields += ['state', 'year']
+    
+    affected_rows = pg_shovel(cnxn, staging[0], table, fields)
+    
+    pg_drop(rawload)
+    pg_drop(staging[0])
+    
+    # affected_rows will evaluate un-True in the case of any troubles
+    if affected_rows:
+        load.success()
+        logger.info('Import complete: %s (%s rows)' % (load, affected_rows))
+        return True
+    else:
+        load.fail()
+        logger.error('Import failed: %s' % load)
+        return False
+
+
 @celery.shared_task(bind=True, ignore_result=True)
 def import_file(self, importqueue_pk, insert_size=5, slice_size=1000):
     load = ImportQueue.objects.get(pk=importqueue_pk)
@@ -270,6 +474,15 @@ def import_file(self, importqueue_pk, insert_size=5, slice_size=1000):
         # this is preferred to a chain so that it doesn't wait to start until after all the signatures are generated
         result_lst = []
         for df in reader:
+            # everything we insert will always have a state and year associated with it
+            # if they aren't already in the frame, add them from the passed values
+            # (which should in turn be from the loadfile object)
+            if 'state' not in df.columns.map(lambda x: str(x).lower()):
+                df['state'] = state
+            
+            if 'year' not in df.columns.map(lambda x: str(x).lower()):
+                df['year'] = year
+            
             cleaned_df = replace_sentinels(df)
             
             if dsabbr == 'CHGS' and not meta_is_long(meta):
@@ -281,8 +494,6 @@ def import_file(self, importqueue_pk, insert_size=5, slice_size=1000):
                     cleaned_df,
                     table,
                     schema,
-                    state,
-                    year, 
                     insert_size
                     )
                 )
@@ -292,7 +503,7 @@ def import_file(self, importqueue_pk, insert_size=5, slice_size=1000):
 
 
 @celery.shared_task
-def chunker(df_slice, table, schema, state, year, insert_size=5):
+def chunker(df_slice, table, schema, insert_size=5):
     """These should be slices far too large to build a SQL string on; e.g. >100"""
     
     r = xrange(0, len(df_slice), insert_size)
@@ -302,9 +513,7 @@ def chunker(df_slice, table, schema, state, year, insert_size=5):
         zip(
             (df_slice[x:x+insert_size] for x in r),
             (table for x in r),
-            (schema for x in r),
-            (state for x in r),
-            (year for x in r),
+            (schema for x in r)
             ),
         10
         ).group()
@@ -316,16 +525,7 @@ def chunker(df_slice, table, schema, state, year, insert_size=5):
 
 @celery.shared_task
 @tx.atomic # turn off autocommit
-def chunk_insert(df_chunk, table, schema, state, year, placeholder_representation="%s"):    
-    # everything we insert will always have a state and year associated with it
-    # if they aren't already in the frame, add them from the passed values
-    # (which should in turn be from the loadfile object)
-    if 'state' not in df_chunk.columns.map(lambda x: str(x).lower()):
-        df_chunk['state'] = state
-    
-    if 'year' not in df_chunk.columns.map(lambda x: str(x).lower()):
-        df_chunk['year'] = year
-        
+def chunk_insert(df_chunk, table, schema, placeholder_representation="%s"):    
     cursor = cnxns[HACHOIR_DB_ENTRY].cursor()
     
     #df_chunk = replace_sentinels(df_chunk)
@@ -340,10 +540,10 @@ def chunk_insert(df_chunk, table, schema, state, year, placeholder_representatio
     except:
         e = sys.exc_info()[0]
         logger.warning('Failed to generate SQL insert statement and parameters list using pyhcup.db.insert_sql(). Raised %s.' % e)
-        logger.warning('df_chunk is %s' % df_chunk)
-        logger.warning('table is %s' % table)
-        logger.warning('schema is %s' % schema)
-        logger.warning('placeholder is %s' % placeholder_representation)
+        #logger.warning('df_chunk is %s' % df_chunk)
+        #logger.warning('table is %s' % table)
+        #logger.warning('schema is %s' % schema)
+        #logger.warning('placeholder is %s' % placeholder_representation)
         return False
     
     
@@ -353,8 +553,8 @@ def chunk_insert(df_chunk, table, schema, state, year, placeholder_representatio
     except:
         e = sys.exc_info()[0]
         logger.warning('Failed to execute SQL insert statement and parameters list using cursor.execute(). Raised %s.' % e)
-        logger.warning('insert_sql is %s' % insert_sql)
-        logger.warning('values is %s' % values)
+        #logger.warning('insert_sql is %s' % insert_sql)
+        #logger.warning('values is %s' % values)
         
     
     tx.commit(using=HACHOIR_DB_ENTRY)
